(ns poalses.jonathan.dialect.dialect-nlp
  (:require [clojure.datafy :refer [datafy]]
            [dk.simongray.datalinguist :as dl]
            [dk.simongray.datalinguist.triple :refer [triple->datalog]])
  (:import [edu.stanford.nlp.coref CorefCoreAnnotations$CorefChainAnnotation]))


(def example
  "Great work, thanks for doing this!
  Having had no previous experience with NLP libraries, I was wondering why I couldn’t get your examples to work. Then I realized that I had to download CoreNLP first from https://stanfordnlp.github.io/CoreNLP/ and add stanford-corenlp-4.4.0/* to the classpath. Everything worked fine after that.
  Is this what you are supposed to do? It wasn’t mentioned in the readme, so I was wondering if I did something wrong here or if it is more obvious to people who have already worked with CoreNLP.")

(def test-sentence-one
  "This is an example of correct language usage. I had an idea about using garlic butter with chicken, hence the food was burnt.")

(def test-sentence-two
  "This is an example of incorrect language usage, due to the word why. I had an idea about using garlic butter with chicken, hence why the food was burnt.")

(def test-sentence-three
  "This is an example of incorrect language usage. I had an idea about using garlic butter with chicken, hence why the food was burnt.")

(def test-sentence-four
  "This is an example of incorrect language usage. I had an idea about using garlic butter with chicken, hence why the food was burnt. This one has an extra sentence to tip the balance to good.")

(def test-sentence-five
  "This is an example of Californian slang. So, like, I had this great idea, where we put like, garlic butter with chicken, it's so rad.")

(def test-sentence-six
  "This is all about like. I like lychees, they're delicious. Sometimes life is like watching paint dry, basically very boring")

(def nlp
  (dl/->pipeline {:annotators ["truecase"
                               "quote"
                               "entitymentions"
                               "parse"
                               "depparse"
                               "lemma"
                               "relation"
                               "tokenize"
                               "coref"
                               "openie"
                               "ner"]
                  :quote      {:extractUnclosedQuotes "true"}}))

;; Word sets that will show a sentence as being of that dialect

(def australian-words #{"incorrect" "why"})

(def scottish-words #{"hence"})

(def american-words #{"like"})

(defn fake-test
  [fake]
  false)

;; Predicate vectors to check a sentence and see if it grammatically matches a dialect

(def australian-predicates [fake-test])

(def scottish-predicates [fake-test])

(def american-predicates [fake-test])


;; Take a sentence and figure out its dialect

(defn detect-sentence-dialect [sentence]
  (let [tokens    (dl/tokens sentence)
        dialects1 (if (some australian-words (dl/text tokens)) :australian (when ((apply some-fn australian-predicates) sentence) :australian))
        dialects2 (if (some scottish-words (dl/text tokens)) :scottish (when ((apply some-fn scottish-predicates) sentence) :scottish))
        dialects3 (if (some american-words (dl/text tokens)) :american (when ((apply some-fn american-predicates) sentence) :american))
        dialects  (remove nil? [dialects1 dialects2 dialects3])]
    (if (empty? dialects) [:standard] dialects)))


;; Take a text sample and separate it into its sentences, then for each sentence find its dialects, and return the most common dialect
;; A sentence can have an indeterminate number of dialects associated with it, as detect-sentence-dialects can return a collection,
;;when no dialect can be detected it defaults to standard. IE if there's a sample with 3 sentences, one reads as scottish,
;;one reads as scottish and australian, and the last reads as nothing, it will return a collection containing 2 scottish keys,
;;one australian key, and one standard key, meaning it would be seen as a scottish sample.

(defn detect-sample-dialect [sample]
  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp sample)))))))))

(def annotated-example
  (delay (nlp example)))

(def sentences
  (delay (dl/sentences @annotated-example)))

(def annotated-example-one
  (delay (nlp test-sentence-one)))

(def sentences-one
  (dl/sentences @annotated-example-one))

(def annotated-example-three
  (delay (nlp test-sentence-three)))

(def sentences-three
  (dl/sentences @annotated-example-three))

(def annotated-example-four
  (delay (nlp test-sentence-four)))

(def sentences-four
  (dl/sentences @annotated-example-four))

(def annotated-example-five
  (delay (nlp test-sentence-five)))

(def sentences-five
  (dl/sentences @annotated-example-five))

(def annotated-example-six
  (delay (nlp test-sentence-six)))

(def sentences-six
  (dl/sentences @annotated-example-six))

(def annotated-example-two
  (delay (nlp test-sentence-two)))

(def sentences-two
  (dl/sentences @annotated-example-two))

(defn show-dependencies []
  (clojure.pprint/pprint (dl/dependency-graph (nth sentences-one 1))))


(comment
  ;; Test every annotator in the pipeline
  (map dl/true-case @sentences)
  (map dl/quotations @sentences)
  (map dl/mentions @sentences)
  (map dl/annotation "relation" @sentences)
  (map dl/constituency-tree @sentences)
  (map dl/constituency-tree sentences-one)
  (map dl/constituency-tree @sentences-two)
  (map dl/dependency-graph @sentences)
  (datafy (dl/dependency-graph (nth sentences-one 1)))
  (datafy (dl/dependency-graph (nth @sentences-two 1)))
  (:leafVertices (bean (dl/dependency-graph (nth @sentences-two 1))))
  (map dl/dependency-graph @sentences-two)
  (bean (dl/dependency-graph @sentences-two))
  (map dl/lemma @sentences)
  (map dl/lemma sentences-one)
  (map dl/lemma @sentences-two)
  (dl/text (dl/tokens (nth sentences-one 1)))
  (map dl/tokens @sentences-two)
  (->> (mapcat dl/triples @sentences) (map triple->datalog))
  (dl/annotation CorefCoreAnnotations$CorefChainAnnotation @annotated-example)

  (show-dependencies)

  (def rats (datafy (dl/dependency-graph (nth sentences-one 1))))


  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp test-sentence-one))))))))
  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp test-sentence-two))))))))
  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp test-sentence-three))))))))
  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp test-sentence-four))))))))
  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp test-sentence-five))))))))
  (first (last (sort-by val (frequencies (flatten (map detect-sentence-dialect (dl/sentences (nlp test-sentence-six))))))))


  (last (vals rats))
  (.getTarget (first (last (vals rats))))
  (bean (first (last (vals rats))))
  (bean (:relation (bean (first (last (vals rats))))))

  (datafy (dl/dependency-graph (nth sentences-one 1)))
  (datafy (dl/dependency-graph (nth sentences-five 1)))
  (datafy (dl/dependency-graph (nth sentences-six 0)))
  (datafy (dl/dependency-graph (nth sentences-six 1)))
  (datafy (dl/dependency-graph (nth sentences-six 2)))

  (bean (:relation (bean (nth (nth (vals (datafy (dl/dependency-graph (nth sentences-five 1)))) 3) 2))))

  ;; This identifies "like" in this instance as being an interjection (filler word)
  (.tag (.getTarget (nth (nth (vals (datafy (dl/dependency-graph (nth sentences-five 1)))) 3) 2)))
  (.word (.getTarget (nth (nth (vals (datafy (dl/dependency-graph (nth sentences-five 1)))) 3) 2)))


  (.getLongName (.getRelation (nth (nth (vals (datafy (dl/dependency-graph (nth sentences-five 1)))) 3) 2)))
  (.getShortName (.getRelation (nth (nth (vals (datafy (dl/dependency-graph (nth sentences-five 1)))) 3) 2)))

  (defn extract-relation [word sentences]
    (for [sentence sentences]
      (let [data-edges (vals (datafy (dl/dependency-graph sentence)))]
        (for [edge data-edges]
          (let [edge-bean (bean edge)]
            edge-bean)))))

  (extract-relation "like" sentences-six)

  (for [sentence sentences-six]
    (let [data-edges (vals (datafy (dl/dependency-graph sentence)))]
      (for [edge data-edges]
        (let [edge-bean (bean edge)]
          edge-bean))))

  (datafy (dl/dependency-graph (nth sentences-six 2)))
  (datafy (first (keys (datafy (dl/dependency-graph (nth sentences-six 2))))))
  (nav (datafy (dl/dependency-graph (nth sentences-six 2))))
  (datafy (:governor (first (map datafy (nth (vals (datafy (dl/dependency-graph (nth sentences-six 0)))) 0)))))


  (.word (.getSource (nth (nth (vals (datafy (dl/dependency-graph (nth sentences-five 1)))) 3) 2)))


  (.word (nth (keys (datafy (dl/dependency-graph (nth sentences-one 1)))) 9))
  (.after (nth (keys (datafy (dl/dependency-graph (nth sentences-one 1)))) 9))
  (.word (nth (keys (datafy (dl/dependency-graph (nth sentences-one 1)))) 10))
  (.word (nth (keys (datafy (dl/dependency-graph (nth sentences-one 1)))) 7))
  (.after (nth (keys (datafy (dl/dependency-graph (nth sentences-one 1)))) 7))

  (.word (nth (keys (datafy (dl/dependency-graph (nth sentences-two 1)))) 9))
  (.after (nth (keys (datafy (dl/dependency-graph (nth sentences-two 1)))) 9))
  (.word (nth (keys (datafy (dl/dependency-graph (nth sentences-two 1)))) 10))
  (.tag (nth (keys (datafy (dl/dependency-graph (nth sentences-two 1)))) 10))
  (.tag (nth (keys (datafy (dl/dependency-graph (nth sentences-two 1)))) 9))
  (.tag (nth (keys (datafy (dl/dependency-graph (nth sentences-two 1)))) 10))
  (.tag (nth (keys (datafy (dl/dependency-graph (nth sentences-two 1)))) 9))


  ;; So two main ways, word use and grammar. Word use is simple enough,
  ;;just check if a word in the sentence matches the expected word

  ;; For example, this returns true if the word "why" is in the given sentence
  (some #(= "why" %) (dl/text (dl/tokens sentence)))

  ;; Then we have grammar, which is a lot harder to check. For instance, a sentence using "like"
  ;;as an interjection should return true when passed to the following code.


  (some #(when (even? %) %) '(1 2 3 4))

  (def something [5647 5858 76 938 62626])

  (reduce + something)

  (str 3837 345 8678)

  (apply str something)

  (map str something)

  (def testfreq '(65 65 65 7 7 5 5 5 5 5 5 5 65 65))
  (def testfreq2 '(65 65 65 7 7 5 5 2 5 5 5 5 5 65 65))

  (detect-sentence-dialect sentences-one)
  (detect-sentence-dialect @sentences-two)


  (detect-sample-dialect test-sentence-one)
  (detect-sample-dialect test-sentence-two)



  (detect-sample-dialect sentences-two)
  (detect-sample-dialect sentences-three)
  (detect-sample-dialect sentences-four)


  (detect-sample-dialect test-sentence-one)
  (detect-sample-dialect test-sentence-two)
  (detect-sample-dialect test-sentence-three)
  (detect-sample-dialect test-sentence-four)

  (first (last (sort-by val (frequencies (map detect-sentence-dialect sentences-one)))))
  (first (last (sort-by val (frequencies (map detect-sentence-dialect @sentences-two)))))
  (first (last (sort-by val (frequencies (map detect-sentence-dialect sentences-three)))))





  (detect-sentence-dialect testfreq)
  (detect-sentence-dialect testfreq2)

  (def testfreq '("a" "a" "a" "b" "b" "c" "c" "c" "c" "a" "a" "d" "d"))

  (frequencies testfreq)

  (first (last (sort-by val (frequencies testfreq))))

  (detect-sentence-dialect (dl/text (dl/tokens (nth sentences-one 1))))
  (detect-sentence-dialect (dl/text (dl/tokens (nth @sentences-two 1))))

  (distinct testfreq)

  (dedupe testfreq)

  (for [someval something]
    (str someval))

  (doseq [someval something]
    (println someval))



  (last (vals rats))
  (.getTarget (first (last (vals rats))))
  (bean (first (last (vals rats))))
  (bean (:relation (bean (first (last (vals rats))))))

  (-> (vals rats)
      (last)
      (first)
      (bean)
      (:relation)
      (bean))


  (-> (vals rats)
      (last)
      (first)
      (.getTarget))

  *e


  (System/currentTimeMillis)

  (spit "test.txt" "testtesttest")

  (spit "test2.txt" "{567 \"test\" 678 767}")

  (slurp "test.txt")
  (read-string (slurp "test2.txt"))

  ;; Don't use eval with arbitrary input
  (def horror (eval (read-string (slurp "test2.txt"))))

  horror


  (dl/text sentences-one)
  (dl/triples sentences-one)
  (dl/triples (dl/dependency-graph (nth sentences-one 1)))

  ;; Datafy the annotations. Retrieves direct annotations for every sentence.
  ;; Keep in mind that `dl/recur-datafy` currently doesn't work in this instance
  ;; and will possibly be removed in a future update:
  ;;   https://github.com/simongray/datalinguist/issues/13
  (into {} (map datafy @sentences))
  (keys (into {} (map datafy @sentences)))
  (:natural-logic/relation-triples (into {} (map datafy @sentences)))
  (:semantic-graph/enhanced-plus-plus-dependencies (into {} (map datafy @sentences)))
  (:semantic-graph/collapsed-dependencies (into {} (map datafy @sentences)))
  (:coref/mentions (into {} (map datafy @sentences)))
  (:tokens (into {} (map datafy @sentences)))
  (def play-map (into {} (map datafy @sentences)))
  (def play-map (into {} (datafy @annotated-example)))

  sentences
  @sentences

  play-map

  (:text play-map)
  #_.)